// ----------------------------------------------------------------------------
// HW definition...
#include <linux/kernel.h>
#include <linux/string.h>
#include "pilot.h"

#define sizeofarray(a) (sizeof(a)/sizeof(a[0]))

static HwOutput outs_v1_1[] = {
  { "5v",    OUT_PWR,  .gpio = { 23, ACTIVE_LOW  } },
  { "3v3",   OUT_PWR,  .gpio = { 22, ACTIVE_LOW  } },
  { "rst1",  OUT_RST,  .gpio = { 5,  ACTIVE_LOW  } },
  { "boot1", OUT_BOOT, .gpio = { 6,  ACTIVE_HIGH } },
  { "rst2",  OUT_RST,  .gpio = { 12, ACTIVE_LOW  } },
  { "boot2", OUT_BOOT, .gpio = { 13, ACTIVE_LOW  } },
  { "red",   OUT_LED,  .gpio = { 26, ACTIVE_HIGH } },
  { "green", OUT_LED,  .gpio = { 20, ACTIVE_HIGH } },
};

static HwInput ins_v1_1[] = {
  { "pwr3",  { 4, ACTIVE_LOW}, "pilot 3v3 toggle", "pilot pwr off" },
  { "pwr5",  {21, ACTIVE_LOW}, "pilot 5v toggle",  "pilot pwr off" },
  { "user1", {18, ACTIVE_LOW}, "pilot btn user1",  "pilot btn user1-long" },
  { "user2", {27, ACTIVE_LOW}, "pilot btn user2",  "pilot btn user2-long" },
  { "OC",    {24, ACTIVE_LOW}, "pilot pwr-fault",  NULL }
};

static HwOutput outs_v2[] = {
  { "5v",    OUT_PWR,  .gpio = { 23, ACTIVE_HIGH } },
  { "3v3",   OUT_PWR,  .gpio = { 24, ACTIVE_HIGH } },
  { "rst1",  OUT_RST,  .gpio = { 5,  ACTIVE_LOW  } },
  { "boot1", OUT_BOOT, .gpio = { 6,  ACTIVE_HIGH } },
  { "rst2",  OUT_RST,  .gpio = { 12, ACTIVE_LOW  } },
  { "boot2", OUT_BOOT, .gpio = { 13, ACTIVE_LOW  } },
  { "red",   OUT_LED,  .gpio = { 26, ACTIVE_HIGH } },
  { "green", OUT_LED,  .gpio = { 20, ACTIVE_HIGH } },
};

static HwInput ins_v2[] = {
  { "pwr",  {21, ACTIVE_LOW}, "pilot pwr toggle", "pilot pwr off"},
  { "user", {19, ACTIVE_LOW}, "pilot btn user",   "pilot btn user-long"},
  { "OC",   {25, ACTIVE_LOW}, "pilot pwr-fault",  NULL}
};

PilotHw pilots[] = {
  { .version = "v2.0",
    .pullups = (1<<18 /*usr.btn*/) | (1<<21 /*5v.btn*/) | (1<<4 /*3v3.btn*/) | (1<<24 /*pwr.ovc*/),
    .ins  = ins_v2,  .insNb  = sizeofarray(ins_v2),
    .outs = outs_v2, .outsNb = sizeofarray(outs_v2),
  },
  { .version = "v1.1",
    .pullups = (1<<18 /*usr0.btn*/) | (1<<27 /*usr1.btn*/) | (1<<21 /*5v.btn*/) | (1<<4 /*3v3.btn*/) | (1<<24 /*pwr.ovc*/),
    .ins  = ins_v1_1,  .insNb  = sizeofarray(ins_v1_1),
    .outs = outs_v1_1, .outsNb = sizeofarray(outs_v1_1),
  },
};

const unsigned char pilotsNb = sizeofarray(pilots);
unsigned char pilotsIdx = 1; // v1.1

extern void setHardware(const char *version) {
  int i;
  for (i=0; i<pilotsNb; i++) {
    if (strcmp(pilots[i].version, version) == 0) {
      pilotsIdx = i;
      return;
    }
  }
  printk(KERN_WARNING "ToBe-Pilot: Setting unknown HW (%s), using %s.\n", version, pilots[pilotsIdx].version);
}
