/** Periodic tick.
 * Maintains a list of functions that are called (polled) periodically
 */
#include <linux/timer.h>
#include "tick.h"

// Button poll interval (100ms)
#define TICK_ITVL (HZ/10)

static struct timer_list timer;

// simple list handling
#define LIST_MAX 2
static tickCallBack list[LIST_MAX] = { NULL, NULL };
unsigned int listNb = 0;

static void tick(struct timer_list *unused) {
  int i;

  mod_timer(&timer, timer.expires + TICK_ITVL);
  for (i=0; i<listNb; i++)
    if (list[i]) list[i]();
}

void tickInit(void) {
  timer_setup(&timer, tick, 0ul);
  timer.expires = jiffies + TICK_ITVL;
  add_timer(&timer);
}

void tickExit(void) {
  del_timer(&timer);
}

void tickAdd(tickCallBack cb) {
  if (listNb < LIST_MAX)
    list[listNb++] = cb;
}

void tickDel(tickCallBack cb) {
  int i;
  for (i=0; i<LIST_MAX; i++)
    if (list[i] == cb) list[i] = NULL;
}
