#include <iostream>
#include <unistd.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include "Socket.h"

using namespace std;

//-----------------------------------------------------------------------------

SocketServer::SocketServer(const char *path) {
  memset(&addr, 0, sizeof(addr));
  addr.sun_family = AF_UNIX;
  strncpy(addr.sun_path, path, sizeof(addr.sun_path)-1);

  sk = socket(AF_UNIX, SOCK_STREAM, 0);
  if (sk <= 0) {
      cerr << "Error: socket creation failed\n";
      return;
  }
  unlink(addr.sun_path); // Remove previous socket file if any ...

  if (bind(sk, (const struct sockaddr *)&addr, sizeof(addr)) < 0) {
      cerr << "Error: socket bind failed\n";
      close(sk);
      sk = 0;
  }
  chmod(addr.sun_path, 0777); // So that  www-data can access socket
}

SocketServer::~SocketServer(void) {
  if (sk > 0) close(sk);
  unlink(addr.sun_path);
}

int SocketServer::run(void) {
  if (sk <=0) return -1;
  if (listen(sk, 5) == -1) {
    cerr << "Error: Socket listen failed\n";
    return -1;
  }

  while (1) {
    int fd = accept(sk, NULL, NULL);
    if (fd == -1) {
      cerr << "Error: Socket accept failed\n";
      return -1; // error counter???
    }

    char buf[32];
    int sz = read(fd, buf, 31);
    if (sz > 0) {
      buf[sz] = 0; // ensure we are NULL terminated
      doCmd(buf, fd);
      close(fd);
      if (strcmp("quit", buf) == 0) break;
    }
  }

  return 0;
}

int SocketServer::send(int fd, const char *data, int len) {
  if (fd > 0) return write(fd, data, len);
  return 0;
}

SocketClient::SocketClient(const char *path) {
  memset(&addr, 0, sizeof(addr));
  addr.sun_family = AF_UNIX;
  strncpy(addr.sun_path, path, sizeof(addr.sun_path)-1);

  sk = socket(AF_UNIX, SOCK_STREAM, 0);
  if (sk <= 0) {
      cerr << "Error: socket creation failed\n";
      return;
  }

  if (connect(sk, (struct sockaddr*)&addr, sizeof(addr)) == -1) {
    close(sk); sk = 0;
    cout << "connect error\n";
    return;
  }
}

SocketClient::~SocketClient() {
  if (sk > 0) close(sk);
}

static char buf[80];

const char *SocketClient::get(const char *cmd) {
  if (sk <= 0) return "";
  int sz = write(sk, cmd, strlen(cmd));
  //cout << "Get : " << cmd << "(" << strlen(cmd) << ")\n";
  do {
    sz = read(sk, buf, sizeof(buf-1));
    if (sz>0) {
      buf[sz] = 0;
      cout << buf;
    }
  } while (sz > 0);

  if (sz == -1) {
    cerr << "read error\n";
    return "";
  }

  //if (sz == 0)
  //  cout << "EOF\n";

  return buf;
}

//-----------------------------------------------------------------------------

#ifdef STANDALONE

class TestSocketServer : public SocketServer {
 public:
  TestSocketServer(const char *path) : SocketServer(path) {}
 protected:
  virtual int doCmd(const char *cmd, int fd) {
    cout << "Received: " << cmd << '\n';
    return send(fd, "Hello", 6);
  }
};

int main(int argc, char **argv) {
  const char *cmd = "Ping";

  if (argc > 1) {
    if (strcmp("-svr", argv[1]) == 0) {
      TestSocketServer srv(sockFile);
      return srv.run();
    }
    else if (strcmp("-quit", argv[1]) == 0)
      cmd ="quit";
  }

  SocketClient client(sockFile);
  client.get(cmd);
}
#endif
