#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <termios.h>
#include <string.h>
#include "Uart.h"

// UART =======================================================================

static void *uartMain(void *ptr) {
  ((Uart*)ptr)->exec();
  return NULL;
}

Uart::Uart(const char *device, int speed, int parity, bool hwFlowCtrl) {
  fd = open(device, O_RDWR | O_NOCTTY | O_SYNC);
  if (fd < 0) {
    fprintf(stderr, "Uart: Error: can't open device %s\n", device);
    return;
  }

  struct termios tty;
  memset (&tty, 0, sizeof tty);
  if (tcgetattr(fd, &tty) != 0) {
    fprintf(stderr, "Uart: init error\n");
    return;
  }
  int baud = B9600;
  switch (speed) {
    //case 9600: baud = B9600;   break;
    case 19200:  baud = B19200;  break;
    case 38400:  baud = B38400;  break;
    case 57600:  baud = B57600;  break;
    case 115200: baud = B115200; break;
  }
  cfsetospeed(&tty, baud);
  cfsetispeed(&tty, baud);
  cfmakeraw(&tty);
  //tty.c_cc[VMIN]  = 0; // Read does not block
  //tty.c_cc[VTIME] = 2; // Read will timeout after 0.2 seconds
  if (hwFlowCtrl)
    tty.c_cflag |= CRTSCTS;
  else
    tty.c_cflag &= ~CRTSCTS;

  if (tcsetattr(fd, TCSANOW, &tty) != 0) {
    fprintf(stderr, "Uart: init error\n");
    return;
  }
  // Start the receive handling thread
  pthread_create(&pth, NULL, uartMain, this);
  printf("Uart: started...\n");
}

Uart::~Uart(void) {
  if (fd > 0) close(fd);
  pthread_cancel(pth);
  pthread_join(pth, NULL);
  printf("Uart: stopped.\n");
}

void Uart::send(const char *data) {
  if (fd > 0)
    write(fd, data, strlen(data));
}

// Receive loop: runs in a thread continuously reading UART
// we read 8 byte buffers since UART driver gives 8bytes, most of the time!
void Uart::exec(void) {
  char tmp[9];

  if (fd < 0) return;

  while (1) {
    ssize_t sz = read(fd, tmp+1, 8);
    if (sz > 0) {
      *tmp = (sz-1) & 0x7; // storing size-1 (3bits)
      received(tmp);
    }
  }
}

//-----------------------------------------------------------------------------
#if 0
class TestUart : public Uart {
 public:
  TestUart() : Uart("/dev/ttyAMA0", 115200, 0, false) {};
 protected:
  void received(const char *tmp) {
    int sz = *tmp++ + 1;
    for (int i=0; i<sz; i++)
      //buf.put(tmp, sz);
      switch (tmp[i]) {
      case 0: printf("<0>"); break;
      case '\n': printf("<NL>\n"); break;
      case '\r': printf("<CR>"); break;
      default:
        if ((tmp[i] > 31) && (tmp[i] <128))
          putchar(tmp[i]);
        else
          printf("<x%d>", tmp[i]);
      break;
    }
    printf("\n");
  }
};

int main(void) {
  TestUart uart;
  uart.exec();
}
#endif
