#ifndef UART_H
#define UART_H

#include <pthread.h>

/** Uart interface class.
 * Provides UART read loop, and UART write support.
 * UART read data is passed as 9-bytes array, 1st byte is (size-1) followed by (up to) 8 data bytes.
 * Uart read gives back 8-bytes most of the time, so we align on that.
 * Implementer should override receive function.
 */
class Uart {
  pthread_t pth;
  int fd;
 public:
  Uart(const char *device, int speed, int parity, bool hwFlowCtrl);
  virtual ~Uart(void);
  void send(const char *data);
  void exec(void);
 protected:
  virtual void received(const char *tmp) = 0;
};
#endif /*UART_H*/
