#!/bin/bash

# rsync -avz ../scripts/ pi@baker:local/pilot

#trap "echo EXIT;  exit" 0
#trap "echo HUP;   exit" 1
#trap "echo CTL-C; exit" 2
#trap "echo QUIT;  exit" 3
#trap "echo ERR;   exit" ERR

function user() {
  case $2 in
    "serial")
      minicom -D $3 &
      ;;

    "enable")
      sudo sh -c "echo run > /sys/devices/tobe/pilot/rst$1"
      ;;

    "disable")
      sudo sh -c "echo reset > /sys/devices/tobe/pilot/rst$1"
      sudo sh -c "echo normal > /sys/devices/tobe/pilot/boot$1"
      ;;

    "reset")
      sudo sh -c "echo reset > /sys/devices/tobe/pilot/rst$1"
      sudo sh -c "echo normal > /sys/devices/tobe/pilot/boot$1"
      sleep 0.5
      sudo sh -c "echo run > /sys/devices/tobe/pilot/rst$1"
      ;;

    "flash")
      sudo sh -c "echo reset > /sys/devices/tobe/pilot/rst$1"
      sudo sh -c "echo bootl > /sys/devices/tobe/pilot/boot$1"
      sleep 0.5
      sudo sh -c "echo run > /sys/devices/tobe/pilot/rst$1"
      ;;

    "")
      cat /sys/devices/tobe/pilot/rst$1
      cat /sys/devices/tobe/pilot/boot$1
      ;;

    "-h")
      echo "pilot <cmd>         - Show MCU state"
      echo "pilot <cmd> reset   - Reset MCU"
      echo "pilot <cmd> flash   - Reset MCU to Bootloader/flash mode"
      echo "pilot <cmd> disable - Keep MCU in reset state (Boot is set to normal)"
      echo "pilot <cmd> enable  - de assert reset"
      echo "pilot <cmd> serial  - Start serial console"
      echo "  with <cmd> usr1|usr2|usrs|stm|esp"
      ;;

    *)
      echo "Unknown command '$2'"
      exit 2
      ;;
  esac
}

function pwr() {
  case "$2" in
    "on"|"off"|"toggle")
      sudo sh -c "echo $2 > /sys/devices/tobe/pilot/$1"
      ;;

    "")
      cat /sys/devices/tobe/pilot/$1
      ;;

    "-h")
      echo "pilot pwr|3v3|5v               - Show power rail state"
      echo "pilot pwr|3v3|5v on|off|toggle - Set power rail state"
      ;;

    *)
      echo "Unknown command '$2'"
      exit 2
      ;;
  esac
}

case "$1" in
  "stm"|"usr1")
    user "1" "$2" "/dev/ttyUSB0"
    ;;

  "esp"|"usr2")
    user "2" "$2" "/dev/ttyAMA0"
    ;;

  "usrs")
    user "1" "$2" "/dev/ttyUSB0"
    user "2" "$2" "/dev/ttyAMA0"
    ;;

  "rst1"|"rst2"|"boot1"|"boot2"|"status")
    if [ "$2" = "" ] ; then
      cat /sys/devices/tobe/pilot/$1
    elif [ "$2" = "-h" ] ; then
      echo "pilot rst1|rst2 [run|reset]      - Show/set reset state"
      echo "pilot boot1|boot2 [normal|flash] - Show/set flash state"
      echo "pilot status [reset|run|ok|err]  - Show/set status state"
    else
      sudo sh -c "echo $2 > /sys/devices/tobe/pilot/$1"
    fi
    ;;

  "pwr")
    pwr "3v3" "$2"
    pwr "5v"  "$2"
    ;;

  "3v3"|"5v")
    pwr "$1" "$2"
    ;;

  "")
    cat /sys/devices/tobe/pilot/stats
    ;;

  # functions called by module
  "init")
    # Ctrl-C sets fail led
    trap "pilot status err; exit" 2

    pilot status run
    pilot pwr off
    pilot usr1 disable
    pilot usr2 disable

    sleep 0.5

    pilot pwr on

    pilot usr1 reset
    pilot usr2 reset

    pilot status ok
    ;;

  "btn")
    trap "pilot status err; exit" 2
    pilot status run
    case "$2" in
      "user1"|"user2"|"user")
        # Add user code here
        pilot usrs reset
        pilot status ok
        ;;

      "user1-long"|"user2-long"|"user-long")
        # add user code here
        pilot usrs flash
        pilot status ok
        ;;

      *)
        echo "Unknown command '$2'"
        pilot status err
        sleep 0.3
        pilot status 0
        exit 2
        ;;
    esac
    ;;

  "pwr-fault")
    pilot status err
    ;;

  "-h")
    echo "pilot stm|esp|usrs|rst1|rst2|boot1|boot2|pwr|3v3|5v|status ..."
    ;;

  *)
    echo "Unknown command '$1'"
    exit 1
    ;;
esac
